@@ -1,14 +1,14 @@
-#  [ANDES] (C)2015-2020 Hantao Cui
+#  [ANDES] (C)2015-2021 Hantao Cui
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 3 of the License, or
 #  (at your option) any later version.
-#
+
 #  File name: param.py
 #  Last modified: 8/16/20, 7:27 PM
 
-from typing import Optional, Union, Callable, List, Tuple
+from typing import Optional, Union, Callable, List, Tuple, Type, Iterable
 
 import math
 import logging
@@ -66,7 +66,10 @@ class BaseParam:
                  info: Optional[str] = None,
                  unit: Optional[str] = None,
                  mandatory: bool = False,
-                 export: bool = True):
+                 export: bool = True,
+                 iconvert: Optional[Callable] = None,
+                 oconvert: Optional[Callable] = None,
+                 ):
         self.name = name
         self.default = default
         self.tex_name = tex_name if (tex_name is not None) else name
@@ -77,6 +80,10 @@ class BaseParam:
 
         self.v = []
         self.property = dict(mandatory=mandatory)
+        self.iconvert = iconvert
+        self.oconvert = oconvert
+        self.vtype = float
+        self.eltype = list
 
     def add(self, value=None):
         """
@@ -91,6 +98,67 @@ class BaseParam:
         -----
         If the value is ``math.nan``, it will set to ``None``.
         """
+        value = self._sanitize(value)
+
+        if isinstance(self.v, list):
+            self.v.append(value)
+        else:
+            self.v = np.append(self.v, value)
+
+    def set(self, pos, attr, value):
+        """
+        Set attributes of the BaseParam class to new values at the given positions.
+
+        Parameters
+        ----------
+        pos : int, list of integers
+            Positions in arrays where the values should be set
+        attr : 'v', 'vin'
+            Name of the attribute to be set
+        value : str, float or list of above
+            New values
+        """
+        if not isinstance(pos, Iterable):
+            pos = [pos]
+
+        if isinstance(self.__dict__[attr], list):
+            for ii, p in enumerate(pos):
+                val = self._sanitize(value[ii])
+                self.__dict__[attr][p] = val
+        else:
+            new_vals = np.zeros_like(value)
+            for ii, p in enumerate(pos):
+                new_vals[ii] = self._sanitize(value[ii])
+            self.__dict__[attr][:] = new_vals
+
+    def set_all(self, attr, value):
+        """
+        Set attributes of the BaseParam class to new values for all positions.
+
+        Parameters
+        ----------
+        attr : 'v', 'vin'
+            Name of the attribute to be set
+        value : list of str, float or int
+            New values
+        """
+        if len(value) != self.n:
+            raise ValueError("Value length does not match parameter numbers")
+
+        if isinstance(self.__dict__[attr], list):
+            for ii, val in enumerate(value):
+                val = self._sanitize(val)
+                self.__dict__[attr][ii] = val
+        else:
+            new_vals = np.zeros_like(value)
+            for ii, val in enumerate(value):
+                new_vals[ii] = self._sanitize(val)
+            self.__dict__[attr][:] = new_vals
+
+    def _sanitize(self, value):
+        """
+        Helper function for sanitizing parameter value.
+        """
 
         if isinstance(value, float) and math.isnan(value):
             value = None
@@ -102,10 +170,7 @@ class BaseParam:
             else:
                 value = self.default
 
-        if isinstance(self.v, list):
-            self.v.append(value)
-        else:
-            self.v = np.append(self.v, value)
+        return value
 
     def get_property(self, property_name: str):
         """
@@ -203,9 +268,14 @@ class IdxParam(BaseParam):
                  mandatory: bool = False,
                  unique: bool = False,
                  export: bool = True,
-                 model: Optional[str] = None):
-        super().__init__(default=default, name=name, tex_name=tex_name, info=info, unit=unit, mandatory=mandatory,
-                         export=export)
+                 model: Optional[str] = None,
+                 iconvert: Optional[Callable] = None,
+                 oconvert: Optional[Callable] = None,
+                 ):
+        super().__init__(default=default, name=name, tex_name=tex_name,
+                         info=info, unit=unit, mandatory=mandatory,
+                         export=export, iconvert=iconvert, oconvert=oconvert,
+                         )
         self.property['unique'] = unique
         self.model = model  # must be a `Model` name for building BackRef - Not checked yet
 
@@ -223,8 +293,11 @@ class NumParam(BaseParam):
     """
     A computational numerical parameter.
 
-    Parameters defined using this class will have their `v` field converted to a NumPy array after adding.
-    The original input values will be copied to `vin`, and the system-base per-unit conversion coefficients
+    Parameters defined using this class will have their `v`
+    field converted to a NumPy array after adding.
+
+    The original input values will be copied to `vin`,
+    and the system-base per-unit conversion coefficients
     (through multiplication) will be stored in `pu_coeff`.
 
     Parameters
@@ -245,6 +318,8 @@ class NumParam(BaseParam):
         Unit of the parameter
     vrange : list, tuple, optional
         Typical value range
+    vtype : type, optional
+        Type of the ``v`` field. The default is ``float``.
 
     Other Parameters
     ----------------
@@ -253,41 +328,50 @@ class NumParam(BaseParam):
     Vn : str
         Name of the parameter for the device base voltage.
     non_zero : bool
-        True if this parameter must be non-zero
-    positive: bool
-        True if this parameter must be positive
+        True if this parameter must be non-zero. `non_zero`
+        can be combined with `non_positive` or `non_negative`.
+    non_positive : bool
+        True if this parameter must be non-positive.
+    non_negative : bool
+        True if this parameter must be non-negative.
     mandatory : bool
-        True if this parameter must not be None
+        True if this parameter must not be None.
     power : bool
         True if this parameter is a power per-unit quantity
-        under the device base
+        under the device base.
+    iconvert : callable
+        Callable to convert input data from excel or others
+        to the internal ``v`` field.
+    oconvert : callable
+        Callable to convert input data from internal type
+        to a serializable type.
     ipower : bool
         True if this parameter is an inverse-power per-unit
-        quantity under the device base
+        quantity under the device base.
     voltage : bool
         True if the parameter is a voltage pu quantity
-        under the device base
+        under the device base.
     current : bool
         True if the parameter is a current pu quantity
-        under the device base
+        under the device base.
     z : bool
         True if the parameter is an AC impedance pu quantity
-        under the device base
+        under the device base.
     y : bool
         True if the parameter is an AC admittance pu quantity
-        under the device base
+        under the device base.
     r : bool
         True if the parameter is a DC resistance pu quantity
-        under the device base
+        under the device base.
     g : bool
         True if the parameter is a DC conductance pu quantity
-        under the device base
+        under the device base.
     dc_current : bool
         True if the parameter is a DC current pu quantity under
-        device base
+        device base.
     dc_voltage : bool
         True if the parameter is a DC voltage pu quantity under
-        device base
+        device base.
 
     """
 
@@ -298,8 +382,12 @@ class NumParam(BaseParam):
                  info: Optional[str] = None,
                  unit: Optional[str] = None,
                  vrange: Optional[Union[List, Tuple]] = None,
+                 vtype: Optional[Type] = float,
+                 iconvert: Optional[Callable] = None,
+                 oconvert: Optional[Callable] = None,
                  non_zero: bool = False,
-                 positive: bool = False,
+                 non_positive: bool = False,
+                 non_negative: bool = False,
                  mandatory: bool = False,
                  power: bool = False,
                  ipower: bool = False,
@@ -314,10 +402,12 @@ class NumParam(BaseParam):
                  export: bool = True,
                  ):
         super(NumParam, self).__init__(default=default, name=name, tex_name=tex_name, info=info,
-                                       unit=unit, export=export)
+                                       unit=unit, export=export, iconvert=iconvert, oconvert=oconvert,
+                                       )
 
         self.property = dict(non_zero=non_zero,
-                             positive=positive,
+                             non_positive=non_positive,
+                             non_negative=non_negative,
                              mandatory=mandatory,
                              power=power,
                              ipower=ipower,
@@ -333,6 +423,7 @@ class NumParam(BaseParam):
         self.pu_coeff = np.ndarray([])
         self.vin = None  # values from input
         self.vrange = vrange
+        self.vtype = vtype
 
     def add(self, value=None):
         """
@@ -346,11 +437,12 @@ class NumParam(BaseParam):
 
         """
 
+        if hasattr(self, 'iconvert') and callable(self.iconvert):
+            value = self.iconvert(value)
+
         # check for math.nan, usually imported from pandas
         if isinstance(value, float) and math.isnan(value):
             value = None
-        elif isinstance(value, str):
-            value = float(value)
 
         # check for mandatory
         if value is None:
@@ -359,45 +451,58 @@ class NumParam(BaseParam):
             else:
                 value = self.default
 
-        # check for non-zero
-        if value == 0.0 and self.get_property('non_zero'):
-            logger.warning(f'Non-zero parameter {self.owner.class_name}.{self.name} corrected to {self.default}')
-            value = self.default
+        if isinstance(value, float):
+            # check for non-zero
+            if value == 0.0 and self.get_property('non_zero'):
+                logger.warning('Non-zero parameter %s.%s corrected to %s',
+                               self.owner.class_name, self.name, self.default)
+                value = self.default
 
-        # check for positive
-        if value is not None and value <= 0.0 and self.get_property('positive'):
-            logger.warning(f'Positive parameter {self.owner.class_name}.{self.name} corrected to {self.default}')
-            value = self.default
+            # check for non-positive
+            if value > 0.0 and self.get_property('non_positive'):
+                logger.warning('Non-Positive parameter %s.%s corrected to %s',
+                               self.owner.class_name, self.name, self.default)
+                value = self.default
+
+            # check for non-negative
+            if value < 0.0 and self.get_property('non_negative'):
+                logger.warning('Non-negative parameter %s.%s corrected to %s',
+                               self.owner.class_name, self.name, self.default)
+                value = self.default
 
         super(NumParam, self).add(value)
 
     def to_array(self):
         """
-        Convert ``v`` to np.ndarray after adding elements.
-        Store a copy if the input in `vin`.
-        Set ``pu_coeff`` to all ones.
+        Converts field ``v`` to the NumPy array type.
+        to enable array-based calculation.
 
-        The conversion enables array-based calculation.
+        Must be called after adding all elements.
+        Store a copy of original input values to field ``vin``.
+        Set ``pu_coeff`` to all ones.
 
         Warnings
         --------
-        After this call, `add` will not be allowed, because data will not be copied over to ``vin``.
+        After this call, `add` will not be allowed to avoid
+        unexpected issues.
         """
 
+        self.v = np.array(self.v, dtype=self.vtype)
+
         # data quality check
         # ----------------------------------------
-        self.v = np.array(self.v, dtype=float)
-
         # NOTE: temporarily disabled due to nested parameters
         # if np.sum(np.isnan(self.v)) > 0:
         #     raise ValueError(f'Param <{self.name} contains NaN.')
 
-        self.v[self.v == np.inf] = 1e8
-        self.v[self.v == -np.inf] = -1e8
+        if self.v.dtype != object:
+            self.v[self.v == np.inf] = 1e8
+            self.v[self.v == -np.inf] = -1e8
         # ----------------------------------------
 
-        self.vin = np.array(self.v, dtype=float)
-        self.pu_coeff = np.ones_like(self.v)
+        self.vin = np.array(self.v, dtype=self.vtype)
+
+        self.pu_coeff = np.ones_like(self.v, dtype=float)
 
     def set_pu_coeff(self, coeff):
         """
@@ -411,7 +516,15 @@ class NumParam(BaseParam):
         coeff : np.ndarray
             An array with the pu conversion coefficients
         """
-        self.pu_coeff = coeff
+        if self.pu_coeff.ndim == 1:
+            self.pu_coeff[:] = coeff
+
+        elif self.pu_coeff.ndim == 2:
+            for ii in range(len(self.pu_coeff)):
+                self.pu_coeff[ii] = coeff[ii]
+        else:
+            raise NotImplementedError("Parameters with ndim > 2 not understood.")
+
         self.v[:] = self.vin * self.pu_coeff
 
     def restore(self):
@@ -476,7 +589,7 @@ class TimerParam(NumParam):
     def is_time(self, dae_t):
         """
         Element-wise check if the DAE time is the same as the parameter value. The current implementation uses
-        `np.isclose`
+        `np.equal`.
 
         Parameters
         ----------
@@ -488,11 +601,13 @@ class TimerParam(NumParam):
         np.ndarray
             The array containing the truth value of if the DAE time is close to the parameter value.
 
-        See Also
-        --------
-        numpy.isclose : See NumPy.isclose for the warning on absolute tolerance
+        Notes
+        -----
+        The previous implementation with `np.isclose` with default `rtol=1e-5` mistakes
+        the immediate pre- and post-event time as in-event when simulation time is greater than 10.
         """
-        return np.isclose(dae_t, self.v)
+
+        return np.equal(dae_t, self.v)
 
 
 class ExtParam(NumParam):
@@ -519,7 +634,7 @@ class ExtParam(NumParam):
                  model: str,
                  src: str,
                  indexer=None,
-                 dtype=float,
+                 vtype=float,
                  allow_none=False,
                  default=0.0,
                  **kwargs):
@@ -527,7 +642,7 @@ class ExtParam(NumParam):
         self.model = model
         self.src = src
         self.indexer = indexer
-        self.dtype = dtype
+        self.vtype = vtype
         self.parent_model = None   # parent model instance
         self.allow_none = allow_none
         self.default = default
@@ -609,6 +724,6 @@ class ExtParam(NumParam):
         """
         Convert to array when d_type is not str
         """
-        if self.dtype == str:
+        if self.vtype == str:
             return
         NumParam.to_array(self)
