@@ -77,7 +77,7 @@ class Node(NodeData, Model):
                        tex_name='V_{dc}',
                        info='voltage magnitude',
                        unit='p.u.',
-                       diag_eps=1e-6,
+                       diag_eps=True,
                        )
 
         self.v.v_str = 'flat_start*1 + ' \
@@ -224,7 +224,7 @@ class Ground(ModelData, Model):
                          info='Ficticious current injection from ground',
                          e_str='u * (v - voltage)',
                          v_str='0',
-                         diag_eps=1e-6,
+                         diag_eps=True,
                          )
         self.v.e_str = '-Idc'
 
@@ -272,7 +272,8 @@ class L(DC2Term):
                         info='Inductance current',
                         unit='p.u.',
                         v_str='0',
-                        e_str='-u * (v1 - v2) / L',
+                        e_str='-u * (v1 - v2)',
+                        t_const=self.L,
                         )
         self.v1.e_str = '-IL'
         self.v2.e_str = '+IL'
@@ -297,7 +298,8 @@ class C(DC2Term):
                         info='Capacitor current',
                         unit='p.u.',
                         v_str='0',
-                        e_str='-u * Idc / C',
+                        e_str='-u * Idc',
+                        t_const=self.C
                         )
         self.Idc = Algeb(tex_name='I_{dc}',
                          info='Current from node 2 to 1',
@@ -305,7 +307,7 @@ class C(DC2Term):
                          v_str='0',
                          e_str='u * (vC - (v1 - v2)) + '
                                '(1 - u) * Idc',
-                         diag_eps=1e-6,
+                         diag_eps=True,
                          )
         self.v1.e_str = '-Idc'
         self.v2.e_str = '+Idc'
@@ -334,8 +336,9 @@ class RLs(DC2Term):
         self.IL = State(tex_name='I_L',
                         info='Inductance current',
                         unit='p.u.',
-                        e_str='u * (v1 - v2 - R * IL) / L',
+                        e_str='u * (v1 - v2 - R * IL)',
                         v_str='(v1 - v2) / R',
+                        t_const=self.L,
                         )
         self.Idc = Algeb(tex_name='I_{dc}',
                          info='Current from node 2 to 1',
@@ -370,8 +373,9 @@ class RCp(DC2Term):
         self.vC = State(tex_name='v_C',
                         info='Capacitor current',
                         unit='p.u.',
-                        e_str='-u * (Idc - vC/R) / C',
+                        e_str='-u * (Idc - vC/R)',
                         v_str='v1 - v2',
+                        t_const=self.C,
                         )
         self.Idc = Algeb(tex_name='I_{dc}',
                          info='Current from node 2 to 1',
@@ -379,7 +383,7 @@ class RCp(DC2Term):
                          e_str='u * (vC - (v1 - v2)) + '
                                '(1 - u) * Idc',
                          v_str='-(v1 - v2) / R',
-                         diag_eps=1e-6,
+                         diag_eps=True,
                          )
         self.v1.e_str = '-Idc'
         self.v2.e_str = '+Idc'
@@ -416,13 +420,15 @@ class RLCp(DC2Term):
                         info='Inductance current',
                         unit='p.u.',
                         v_str='0',
-                        e_str='u * vC / L',
+                        e_str='u * vC',
+                        t_const=self.L,
                         )
         self.vC = State(tex_name='v_C',
                         info='Capacitor current',
                         unit='p.u.',
-                        e_str='-u * (Idc - vC/R - IL) / C',
+                        e_str='-u * (Idc - vC/R - IL)',
                         v_str='v1 - v2',
+                        t_const=self.C,
                         )
         self.Idc = Algeb(tex_name='I_{dc}',
                          info='Current from node 2 to 1',
@@ -430,7 +436,7 @@ class RLCp(DC2Term):
                          e_str='u * (vC - (v1 - v2)) + '
                                '(1 - u) * Idc',
                          v_str='-(v1 - v2) / R',
-                         diag_eps=1e-6,
+                         diag_eps=True,
                          )
         self.v1.e_str = '-Idc'
         self.v2.e_str = '+Idc'
@@ -459,8 +465,9 @@ class RCs(DC2Term):
         self.vC = State(tex_name='v_C',
                         info='Capacitor current',
                         unit='p.u.',
-                        e_str='-u * Idc / C',
+                        e_str='-u * Idc',
                         v_str='v1 - v2',
+                        t_const=self.C,
                         )
         self.Idc = Algeb(tex_name='I_{dc}',
                          info='Current from node 2 to 1',
@@ -468,7 +475,7 @@ class RCs(DC2Term):
                          e_str='u * (vC - (v1 - v2) - Idc * R) + '
                                '(1 - u) * Idc',
                          v_str='-(v1 - v2) / R',
-                         diag_eps=1e-6,
+                         diag_eps=True,
                          )
         self.v1.e_str = '-Idc'
         self.v2.e_str = '+Idc'
@@ -504,21 +511,23 @@ class RLCs(DC2Term):
         self.IL = State(tex_name='I_L',
                         info='Inductance current',
                         unit='p.u.',
-                        e_str='u * (v1 - v2 - R * IL - vC) / L',
+                        e_str='u * (v1 - v2 - R * IL - vC)',
                         v_str='0',
+                        t_const=self.L,
                         )
         self.vC = State(tex_name='v_C',
                         info='Capacitor current',
                         unit='p.u.',
-                        e_str='u * IL / C',
+                        e_str='u * IL',
                         v_str='v1 - v2',
+                        t_const=self.C,
                         )
         self.Idc = Algeb(tex_name='I_{dc}',
                          info='Current from node 2 to 1',
                          unit='p.u.',
                          e_str='-IL - Idc',
                          v_str='0',
-                         diag_eps=1e-6,
+                         diag_eps=True,
                          )
         self.v1.e_str = '-Idc'
         self.v2.e_str = '+Idc'
